package ui.internal;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.Stroke;
import java.util.HashSet;
import java.util.Set;

import javax.swing.JPanel;



import ui.Action;



public class MazeViewPanel extends JPanel {

	private Maze myMaze;

	private int nodeLength = 40; // indicates the size of the square box to be displayed
	private float wallWidth = 7;
	
	public void setCellSize(int nodeLength) {
		this.nodeLength = nodeLength;
		repaint();
	}
	public void setWallWidth(float wallWidth) {
		this.wallWidth = wallWidth;
	}
	
	protected int getNodeLength() {
		return nodeLength;
	}
	
	protected float getWallWidth() {
		return wallWidth;
	}
	
	protected Maze getMaze() {
		return myMaze;
	}
	
	public void setMaze(Maze maze) {
		this.myMaze = maze;
		this.setVisible(true);
		this.repaint();
	}

	public MazeViewPanel() {
		GridLayout jGridPanelLayout = new GridLayout(15, 15);
		jGridPanelLayout.setRows(10);
		jGridPanelLayout.setColumns(10);
		setLayout(jGridPanelLayout);
		setLocation(230, 50);
		setSize(10, 10);
		this.setVisible(false);
	}

	public void paintComponent(Graphics g) {
		super.paintComponent(g);

		Graphics2D g2 = (Graphics2D) g;
		setSize(myMaze.getWidth() * nodeLength + 1, myMaze.getHeight()
				* nodeLength + 1);
		
		drawValues(g2);
		
		drawGoals(g2);
		drawTeleports(g2);

		plotMaze(g2);

		drawWalls(g2);

		drawRewards(g2);
		drawActions(g2);
	}
	
	protected void drawValues(Graphics2D g2d) {
		
	}
	
	protected void drawActions(Graphics2D g2d) {
		
	}
	
	protected void drawText(Graphics2D g, int x, int y, Color c, String text) {
		Color oldColor = g.getColor();
		g.setColor(c);
		
		g.drawString(text, x * nodeLength
				+ wallWidth / 2 + 2, (y + 1) * nodeLength - wallWidth
				/ 2 - 2);
		
		g.setColor(oldColor);
	}
	

	protected void drawRewards(Graphics2D g2d) {
		for (int x = 0; x < myMaze.getWidth(); x++) {
			for (int y = 0; y < myMaze.getHeight(); y++) {
				drawText(g2d, x, y, Color.BLACK, myMaze.getReward(x, y)+"");
			}
		}
	}

	protected void plotMaze(Graphics2D g) {
		final int startx = 0;
		final int starty = 0;

		for (int i = 0; i <= nodeLength * myMaze.getWidth(); i = i + nodeLength)
			g.drawLine(i + startx, starty, i + startx, starty
					+ (nodeLength * myMaze.getHeight()));

		for (int i = 0; i <= nodeLength * myMaze.getHeight(); i = i
				+ nodeLength)
			g.drawLine(startx, i + starty, startx
					+ (nodeLength * myMaze.getWidth()), i + starty);
	}

	protected void drawWalls(Graphics g) {

		int aX, aY, bX, bY; // start and end points of the wall

		int height = myMaze.getHeight();
		int width = myMaze.getWidth();

		Set<ResourceWall> walls = new HashSet<ResourceWall>();

		for (int x = 0; x < width; x++) {
			for (int y = 0; y < height; y++) {
				for (Action action : Action.values()) {
					if (myMaze.isWall(x, y, action)) {
						ResourceWall w;

						if (action.equals(Action.GO_SOUTH)) {
							w = new ResourceWall(x, y, ResourceAction.DOWN);
							walls.add(w);
						}
						if (action.equals(Action.GO_NORTH)) {
							w = new ResourceWall(x, y, ResourceAction.UP);
							walls.add(w);
						}
						if (action.equals(Action.GO_WEST)) {
							w = new ResourceWall(x, y, ResourceAction.LEFT);
							walls.add(w);
						}

						if (action.equals(Action.GO_EAST)) {
							w = new ResourceWall(x, y, ResourceAction.RIGHT);
							walls.add(w);
						}

					}
				}
			}
		}

		Graphics2D g2d = (Graphics2D) g;
		Stroke oldStroke = g2d.getStroke();
		BasicStroke lineStyle = new BasicStroke(wallWidth,
				BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);
		g2d.setStroke(lineStyle);

		for (ResourceWall w : walls) {
			int nodeX = w.x;
			int nodeY = w.y;
			switch (w.dir) {
			case ResourceWall.UP:
				aX = nodeX * nodeLength;
				bX = (nodeX + 1) * nodeLength;
				aY = (nodeY) * nodeLength;
				bY = (nodeY) * nodeLength;
				g2d.drawLine(aX, aY, bX, bY);
				break;
			case ResourceWall.DOWN:
				aX = nodeX * nodeLength;
				bX = (nodeX + 1) * nodeLength;
				aY = (nodeY + 1) * nodeLength;
				bY = (nodeY + 1) * nodeLength;
				g2d.drawLine(aX, aY, bX, bY);
				break;
			case ResourceWall.RIGHT:
				aX = (nodeX + 1) * nodeLength;
				bX = (nodeX + 1) * nodeLength;
				aY = (nodeY) * nodeLength;
				bY = (nodeY + 1) * nodeLength;
				g2d.drawLine(aX, aY, bX, bY);
				break;
			case ResourceWall.LEFT:
				aX = (nodeX) * nodeLength;
				bX = (nodeX) * nodeLength;
				aY = (nodeY + 1) * nodeLength;
				bY = (nodeY) * nodeLength;
				g2d.drawLine(aX, aY, bX, bY);
				break;
			}
		}

		g2d.setStroke(oldStroke);
	}

	protected void drawGoals(Graphics2D g2d) {
		Color goalColor = new Color(252, 139, 37);
		
		for (int x = 0; x < myMaze.getWidth(); x++) {
			for (int y = 0; y < myMaze.getHeight(); y++) {
				if (myMaze.isAbsorbingGoal(x, y)) {
					fillCellByColor(g2d, x, y, goalColor);				
				}
			}
		}
	}
	
	
	protected void fillCellByColor(Graphics2D g, int x, int y, Color c) {
		Color oldColor = g.getColor();
		
		int xCorner = x * nodeLength;
		int yCorner = y * nodeLength;
		
		g.setColor(c);
		g.fillRect(xCorner, yCorner, nodeLength, nodeLength);
		g.setColor(oldColor);
		
	}

	protected void drawTeleports(Graphics2D g) {
		Color teleportColor = new Color(100, 139, 37);

		for (int x = 0; x < myMaze.getWidth(); x++) {
			for (int y = 0; y < myMaze.getHeight(); y++) {
				if (myMaze.isTeleportState(x, y)) {
					fillCellByColor(g, x, y, teleportColor);
				}
			}
		}
	}

}
